

(function($) {

    var Fusion = function(element, options){
		//Defaults are below
		var settings = $.extend({}, $.fn.fusion.defaults, options);

        //Useful variables. Play carefully.
        var vars = {
            width: 0,
            aside: 0, //the width of the aside bar
            minHeight: 0, //the min height when using aside bar
            current: 0,
            current2: 0,
            play: false,
            interval: 'none',
            top: 0 //the height of the top bar
        };
    
        //Get the fusion
        var fusion = $(element);
        if(settings.effect == 'vertical'){
        	fusion.addClass('vertical');
        }else if(settings.effect == 'horizontal'){
        	fusion.addClass('horizontal');
        }else{
        	fusion.addClass('mixed');
        }

        if(settings.bar == 'aside')fusion.addClass('aside');
		vars.width = fusion.find('div.content').outerWidth();
		
		//Find our fusion children 
		var navbar = fusion.children('ul');
		var li = fusion.children('ul').find('li');
		var triggers = fusion.children('ul').find('a');
		triggers.first().addClass('selected');
		var section = fusion.find('section').first();
		var contents = section.find('div.content');

		if(settings.effect == 'vertical' && settings.autoHeight==false){
			var maxHeight = 0;

			for(var i=0; i<contents.length; i++){
				var objHeight = contents.eq(i).outerHeight();

				if(objHeight>maxHeight)maxHeight=objHeight;
			}

			for(var i=0; i<contents.length; i++){
				contents.eq(i).css({
					'height': maxHeight+'px'
				});
			}

			var extra = navbar.outerHeight();
			if(settings.bar == 'aside')extra=0;

			fusion.css({
				'height': maxHeight+extra+'px'
			});
		}else if(settings.effect == 'mixed' && settings.autoHeight==false){
			var maxHeight = 0;

			for(var i=0; i<contents.length; i++){
				var objHeight = contents.eq(i).outerHeight();

				if(objHeight>maxHeight)maxHeight=objHeight;
			}

			for(var i=0; i<contents.length; i++){
				contents.eq(i).css({
					'height': maxHeight+'px'
				});
			}

			var extra = navbar.outerHeight();
			if(settings.bar == 'aside')extra=0;

			fusion.css({
				'height': maxHeight+extra+'px'
			});
		}

		fusion.css({
			'opacity' : '1'
		}).addClass('loaded');

		if(settings.bar != 'aside'){
			navbar.css({
				'position':'absolute',
				'width': '100%'
			});
		}

		//controles
		if(settings.showControls || settings.showPlay){
			cleft = '<a href=# class=left><</a>';
			cright = '<a href=# class=right>></a>';
			playBtn = '<a href=# class=play>&#9658;</a>';

			if(!settings.showControls){
				cleft = '';
				cright = '';
			}

			if(!settings.showPlay){
				playBtn = '';
			}


			if(settings.bar == 'aside'){
				$('<div class=controls>'+cleft+playBtn+cright+'</div>').appendTo(navbar).css({
					'top' : '20px',
					'right' : -contents.outerWidth()+15+'px'
				});
			}else{
				$('<div class=controls>'+cleft+playBtn+cright+'</div>').appendTo(navbar).css({
					'top' : navbar.outerHeight()+20,
					'right' : '15px'
				});
			}
		}

		if(settings.bar == 'aside'){
			fusion.css({
				'width': "+="+navbar.outerWidth()+"px"
			});
			section.css('margin-left', navbar.outerWidth());

			vars.aside = navbar.outerWidth();

			vars.minHeight = li.length * li.outerHeight();

			for(var i=0; i<contents.length; i++){
				var objHeight = contents.eq(i).outerHeight();

				if(objHeight<vars.minHeight)contents.eq(i).css({
					'height':vars.minHeight+'px'
				});
			}
		}else{
			section.css('margin-top', navbar.outerHeight());

			vars.top = navbar.outerHeight();
		}

		if(settings.autoHeight){
			var lineHeightFix = 0;

			if ($.trim(contents.eq(0).text()) == ''){//If there is no text (only images) then no line-height
				lineHeightFix = 6;
			}

			var navbarHeight = navbar.outerHeight();
			if(settings.bar == 'aside')navbarHeight = 0;

			var height = (navbarHeight+contents.eq(0).outerHeight()-lineHeightFix);
			if(height<vars.minHeight)height = vars.minHeight;

			//Fix Height, it depends of its content
			fusion.css({
				'height': height+'px'
			});
		}

       	//When user clicks on tabs
       	triggers.on(settings.eventTrigger, function(evt){
			evt.preventDefault();

			var index = triggers.index(this);

			if(vars.play){
				restartInterval(triggers, fusion, section, contents, settings, vars, 'none')
			}

			runSlider(triggers, fusion, section, contents, settings, vars, index);
		});

       	//CONTROLS -->

       	// GO RIGHT
		fusion.find('.right').on('click', function(evt){
			evt.preventDefault();

			var next = vars.current+1;
			if(next>=contents.length){
				next = 0;
			}

			if(vars.play){
				restartInterval(triggers, fusion, section, contents, settings, vars, 'none')
			}

			runSlider(triggers, fusion, section, contents, settings, vars, next);
		});

		//GO LEFT
		fusion.find('.left').on('click', function(evt){
			evt.preventDefault();

			var next = vars.current-1;
			if(next<0){
				next = contents.length-1;
			}

			if(vars.play){
				restartInterval(triggers, fusion, section, contents, settings, vars, 'none')
			}

			runSlider(triggers, fusion, section, contents, settings, vars, next);
		});

		fusion.find('.play').on('click', function(evt){
			evt.preventDefault();

			$this = $(this);
			if(!vars.play){
				vars.play = true;
				$this.html('| |').css({
					'padding' : '5px 7px 5px 7px'
				});
				vars.interval = setInterval(function(){
					runSlider(triggers, fusion, section, contents, settings, vars, 'none')
				}, settings.playInterval);
			}else{
				vars.play = false;
				$this.html('&#9658;').css({
					'padding': '5px 5px 5px 6px'
				});

				vars.interval = window.clearInterval(vars.interval);
			}

		});

		//Keyboard Navigation
        if(settings.keyboardNav){
            $(window).keydown(function(event){
                //Left
                if(event.keyCode == '37'){
                    var next = vars.current-1;
					if(next<0){
						next = contents.length-1;
					}

					if(vars.play){
						restartInterval(triggers, fusion, section, contents, settings, vars, 'none')
					}

					runSlider(triggers, fusion, section, contents, settings, vars, next);
                }
                //Right
                if(event.keyCode == '39'){
                    var next = vars.current+1;
					if(next>=contents.length){
						next = 0;
					}

					if(vars.play){
						restartInterval(triggers, fusion, section, contents, settings, vars, 'none')
					}

					runSlider(triggers, fusion, section, contents, settings, vars, next);
                }
            });
        }

		if(settings.autoPlay){
			if(!vars.play){
				vars.play = true;
				fusion.find('.controls').find('.play').html('| |').css({
					'padding' : '5px 7px 5px 7px'
				});
				vars.interval = setInterval(function(){
					runSlider(triggers, fusion, section, contents, settings, vars, 'none')
				}, settings.playInterval);
			}
		}

		var restartInterval = function(triggers, fusion, section, contents, settings, vars){
			vars.interval = window.clearInterval(vars.interval);
			vars.interval = setInterval(function(){
				runSlider(triggers, fusion, section, contents, settings, vars, 'none')
			}, settings.playInterval);
		}

		var runSlider = function(triggers, fusion, section, contents, settings, vars, index){
			if(index!='none'){//If the method was not call by the play button
				vars.current2 = vars.current;
				vars.current = index;
			}else{
				vars.current2 = vars.current;
				var nxt = vars.current+1;
				if(nxt>=contents.length){
					nxt = 0;
				}

				vars.current = nxt;
			}	

			//Add class of selected to the navbar
			triggers.eq(vars.current).addClass('selected').parent('li').siblings('li').find('a').removeClass('selected');

			if(settings.autoHeight){
				var lineHeightFix = 0;

				if ($.trim(contents.eq(vars.current).text()) == ''){//If there is no text (only images) then no line-height
					lineHeightFix = 6;
				}

				var navbarHeight = navbar.outerHeight();
				if(settings.bar == 'aside')navbarHeight = 0;

				var height = navbarHeight+contents.eq(vars.current).outerHeight()-lineHeightFix;
				if(height<vars.minHeight)height = vars.minHeight;

				//Fix Height, it depends of its content
				fusion.stop();
				fusion.animate({
					'height': height+'px'
				}, settings.animSpeed);
			}

			section.stop();

			if(settings.effect == 'vertical'){//VERTICAL ANIMATION
				var totalHeightBefore = 0;

				for(var i=0; i<vars.current; i++){
					totalHeightBefore+=contents.eq(i).outerHeight();
				}

				section.animate({
					'margin-top' : -totalHeightBefore+vars.top
				}, settings.animSpeed);
			}else if(settings.effect == 'horizontal'){//HORIZONTAL ANIMATION
				section.animate({
					'margin-left' : -vars.current*vars.width+vars.aside
				}, settings.animSpeed);
			}else if(settings.effect == 'mixed'){
				contents.css({
					'z-index': '-1'
				});

				contents.eq(vars.current2).css({
					'z-index': '99'
				});

				objShow = contents.eq(vars.current);

				if(objShow.data('effect') == 'fade'){
					objShow.css({
						'z-index': '100'
					}).hide().fadeIn(settings.animSpeed);
				}else if(objShow.data('effect') == 'slide'){
					objShow.css({
						'z-index': '100'
					}).hide().slideDown(settings.animSpeed);
				}else if(objShow.data('effect') == 'show'){
					objShow.css({
						'z-index': '100'
					}).hide().show(settings.animSpeed);
				}else if(objShow.data('effect') == 'fromTop'){
					objShow.css({
						'z-index': '100',
						'top': -objShow.outerHeight()
					})
					.animate({
						'top': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'fromBottom'){
					objShow.css({
						'z-index': '100',
						'top': section.outerHeight()
					})
					.animate({
						'top': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'fromRight'){
					objShow.css({
						'z-index': '100',
						'left': objShow.outerWidth()
					})
					.animate({
						'left': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'fromLeft'){
					objShow.css({
						'z-index': '100',
						'left': -objShow.outerWidth()
					})
					.animate({
						'left': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'fromTopRight'){
					objShow.css({
						'z-index': '100',
						'top': -objShow.outerHeight(),
						'left': objShow.outerWidth()
					})
					.animate({
						'top': '0px',
						'left': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'fromBottomRight'){
					objShow.css({
						'z-index': '100',
						'top': section.outerHeight(),
						'left': objShow.outerWidth()
					})
					.animate({
						'top': '0px',
						'left': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'fromTopLeft'){
					objShow.css({
						'z-index': '100',
						'top': -objShow.outerHeight(),
						'left': -objShow.outerWidth()
					})
					.animate({
						'top': '0px',
						'left': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'fromBottomLeft'){
					objShow.css({
						'z-index': '100',
						'top': section.outerHeight(),
						'left': -objShow.outerWidth()
					})
					.animate({
						'top': '0px',
						'left': '0px'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'rotateRight'){
					objShow.css({
						'z-index': '100'
					})
					.rotate('-180deg')
					.animate({
						rotate: '0deg'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'rotateLeft'){
					objShow.css({
						'z-index': '100'
					})
					.rotate('180deg')
					.animate({
						rotate: '0deg'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'zoomIn'){
					objShow.css({
						'z-index': '100'
					})
					.scale('2')
					.animate({
						scale: '1'
					},settings.animSpeed);
				}else if(objShow.data('effect') == 'zoomOut'){
					objShow.css({
						'z-index': '100'
					})
					.scale('.3')
					.animate({
						scale: '1'
					},settings.animSpeed);
				}else{
					objShow.css({
						'z-index': '100'
					}).hide().fadeIn(settings.animSpeed);
				}

			}

		};
        
		
		return this;

    };//END OF FUSION OBJECT
        
    $.fn.fusion = function(options) {
    
        return this.each(function(key, value){
            var element = $(this);
            // Return early if this element already has a plugin instance
            if (element.data('fusion')) return element.data('fusion');
            // Pass options to plugin constructor
            var fusion = new Fusion(this, options);
            // Store plugin object in this element's data
            element.data('fusion', fusion);
        });

	};
	
	//Default settings
	$.fn.fusion.defaults = {
		effect: "horizontal",
		animSpeed: 700,
		eventTrigger: 'click',
		autoHeight: true,
		bar: 'top',
		showControls: true,
		showPlay: true,
		playInterval: 2000,
		autoPlay: true,
		keyboardNav: true
	};
	
	$.fn._reverse = [].reverse;
	
})(jQuery);